/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker;

import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;

import java.beans.PropertyVetoException;

import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JLayeredPane;
import javax.swing.JOptionPane;

import jspritemaker.controller.MainFrameMenu;

import jspritemaker.model.Sprite;

import jspritemaker.view.SpriteDialog;
import jspritemaker.view.SpriteManager;
import jspritemaker.view.SpriteView;
import jspritemaker.view.ToolChooser;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class JSpriteMaker implements WindowListener {
	private JFrame frame;
	private SpriteManager views;
	private ToolChooser toolView;
	private boolean toolVisible;
	
	// class constants	
	public final static String APP_NAME    = "JSpriteMaker 68k";
	public final static String APP_VERSION = "0.5.1 build 19";
	
	public final static int WINDOW_WIDTH   = 775;
	public final static int WINDOW_HEIGHT  = 550;
	
	/* constructors */
	
	public JSpriteMaker() {
		this.frame = new JFrame(APP_NAME);
		this.views = new SpriteManager();
		
		// initialize instance variables
		toolVisible = false;
		
		// initialize the frame
		this.initFrame();
		
		// when the frame is complete, display it
		frame.setVisible(true);
	}
	
	/* private methods */
	
	private void initFrame() {
		// create the menu bar
		frame.setJMenuBar(new MainFrameMenu(this));
		
		// add the event listeners
		frame.addWindowListener(this);
		
		// setup frame options
		frame.setSize(WINDOW_WIDTH,WINDOW_HEIGHT);
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		
		// add JDesktopPane
		frame.setContentPane(new JDesktopPane());
	}
	
	private void addNewSprite(int width, int height, int calc, String light, String dark, String mask) {
		SpriteView sprite = views.createNewSprite(width,height,calc,light,dark,mask);
		
		frame.getContentPane().add(sprite);
		
		// make the sprite the active sprite
		try {
			sprite.moveToFront();
			sprite.setSelected(true);
		} catch (PropertyVetoException e) {
			// what would we do if this ever happened...
		}
	}
	
	/* public methods */
	
	public JFrame getFrame() {
		return this.frame;
	}
	
	public SpriteManager getView() {
		return this.views;
	}
	
	// menu methods
	public void fileNew() {
		SpriteDialog dlg = new SpriteDialog(getFrame(),null,SpriteDialog.NEW_SPRITE_DIALOG);
		
		if (dlg.isValid() == JOptionPane.OK_OPTION) {
			this.addNewSprite(dlg.getWidth(),
					  dlg.getHeight(),
					  dlg.getCalc(),
			                  dlg.getLightPlaneName(),
			                  dlg.getDarkPlaneName(),
			                  dlg.getMaskName());
		}
	}
	
	public void fileExit() {
		int result;
		
		result = JOptionPane.showConfirmDialog(frame,"Really Quit?",APP_NAME,
				JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE);
		
		if (result == JOptionPane.YES_OPTION) {
			System.exit(0);
		}
	}
	
	public void editUndo() {
		Sprite model = this.views.getCurrentSpriteModel();
		
		if (model != null) {
			model.undo();
		}
	}
	
	public void editRedo() {
		Sprite model = this.views.getCurrentSpriteModel();
		
		if (model != null) {
			model.redo();
		}
	}
	
	public void editCopy() {
		SpriteView sprite = views.getCurrentSprite();
		
		if (sprite != null) {
			sprite.copy();
		}
	}
	
	public void editSelectAll() {
		SpriteView sprite = views.getCurrentSprite();
		
		if (sprite != null) {
			sprite.selectAll();
		}
	}
	
	public void spriteZoom(int zoom) {
		SpriteView sprite = this.views.getCurrentSprite();
		
		// if a sprite exists
		if (sprite != null) {
			sprite.setZoom(zoom);
		}
	}
	
	public void spriteZoom() {
		SpriteView sprite = this.views.getCurrentSprite();
		String ZOOM_ERROR = "Invalid Zoom Selection";
		String zoomStr = null;
		int zoom;
		boolean done = false;
		
		while (!done) {
			if (sprite != null) {
				zoomStr = (String)JOptionPane.showInputDialog(frame,
					"Zoom Selection [1-50]: ","Set Arbitrary Zoom",
					JOptionPane.QUESTION_MESSAGE,null,null,
					"" + sprite.getZoom());
			}
		
			if (zoomStr != null) {
				try {
					zoom = Integer.parseInt(zoomStr);
			
					if (zoom >= SpriteView.MIN_ZOOM && zoom <= SpriteView.MAX_ZOOM) {
						spriteZoom(zoom);
						done = true;
					} else {
						JOptionPane.showMessageDialog(frame,
							"Zoom must be between " + SpriteView.MIN_ZOOM +
							" and " + SpriteView.MAX_ZOOM,
							ZOOM_ERROR,JOptionPane.ERROR_MESSAGE);
					}
				} catch (NumberFormatException e) {
					JOptionPane.showMessageDialog(frame,
						"Zoom factor must be an integer",ZOOM_ERROR,
						JOptionPane.ERROR_MESSAGE);
				}
			} else {
				done = true;
			}
		}
	}
	
	public void spriteResize() {
		SpriteView view = this.views.getCurrentSprite();
		Sprite model = this.views.getCurrentSpriteModel();
		SpriteDialog dlg;
		
		if ((model != null) && (view != null)) {
			dlg = new SpriteDialog(getFrame(),model,SpriteDialog.RESIZER_DIALOG);
			
			if (dlg.isValid() == JOptionPane.OK_OPTION) {
				model.resize(dlg.getWidth(),dlg.getHeight());
				model.notifyObservers();
				view.resize();
			}
		}
	}
	
	public void spriteOptions() {
		SpriteView view = this.views.getCurrentSprite();
		Sprite model = this.views.getCurrentSpriteModel();
		SpriteDialog dlg;
		
		if ((model != null) && (view != null)) {
			dlg = new SpriteDialog(getFrame(),model,SpriteDialog.PROPERTIES_DIALOG);

			if (dlg.isValid() == JOptionPane.OK_OPTION) {
				model.setProperties(dlg.getWidth(),dlg.getHeight(),dlg.getCalc(),
						    dlg.getLightPlaneName(),
						    dlg.getDarkPlaneName(),
						    dlg.getMaskName());
				model.notifyObservers();
				view.resize();
			}
		}
	}
	
	public void viewViewToolkit() {
		if (toolVisible) {
			toolView.setVisible(false);
			frame.getContentPane().remove(toolView);
			toolView.dispose();
			
			toolVisible = false;
		} else {
			toolView = new ToolChooser(this.views);
			toolView.setVisible(true);
			((JLayeredPane)frame.getContentPane()).add(toolView,JLayeredPane.MODAL_LAYER);
			
			toolVisible = true;
		}
	}
	
	public void helpAbout() {
		JOptionPane.showMessageDialog(frame,APP_NAME + " " + APP_VERSION + 
			"\nCopyright (C) 2002 John David Ratliff", "About " + APP_NAME +
			"...",JOptionPane.INFORMATION_MESSAGE);
	}
	
	/* event listeners */
	
	// window listener interface
	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	
	public void windowClosing(WindowEvent e) {
		fileExit();
	}
	
	/* main method */
	
	public static void main(String[] args) {
		JSpriteMaker app = new JSpriteMaker();
	}
}
