/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.controller;

import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.InputEvent;

import javax.swing.event.MouseInputListener;

import jspritemaker.model.CompoundSpriteEdit;
import jspritemaker.model.Sprite;
import jspritemaker.model.SpriteEdit;

import jspritemaker.toolkit.SpriteToolkit;

import jspritemaker.view.SpriteManager;
import jspritemaker.view.SpriteView;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class SpritePanelInput implements MouseListener, MouseInputListener {
	private SpriteManager view;
	
	// mouse position variables
	private int mouseX, mouseY;
	private int oldX, oldY;
	
	// sprite edit variables
	private CompoundSpriteEdit compoundEdit;
	private boolean changing;
	
	// class constants
	public final static int LEFT_BUTTON    = 0;
	public final static int MIDDLE_BUTTON  = 1;
	public final static int RIGHT_BUTTON   = 2;
	
	public final static int[] BUTTON_MASK  = {InputEvent.BUTTON1_MASK,
			InputEvent.BUTTON2_MASK,InputEvent.BUTTON3_MASK};
	
	/* constructors */
	
	public SpritePanelInput(SpriteManager view) {
		this.view = view;
		
		// initialize variables
		this.compoundEdit = null;
		this.changing = false;
	}
	
	/* private methods */
	
	private int getZoom() {
		return this.view.getCurrentSprite().getZoom();
	}
	
	private boolean isChanging() {
		return this.changing;
	}
	
	public void setChanging(boolean changing) {
		this.changing = changing;
	}
	
	public void beginChange() {
		this.compoundEdit = new CompoundSpriteEdit();
		setChanging(true);
	}
	
	public void endChange() {
		Sprite model = view.getCurrentSpriteModel();
		
		setChanging(false);
		this.compoundEdit.end();
		model.addToUndoStack(this.compoundEdit);
		model.setLastEdit(this.compoundEdit);
	}
	
	// the point tool is the brush/pen/pencil tool of many paint applications
	// drag the mouse with the left button and watch pixels change color
	private void pointTool() {
		SpriteView view = this.view.getCurrentSprite();
		Sprite model = this.view.getCurrentSpriteModel();
		SpriteEdit edit = new SpriteEdit();
		
		if ((mouseX != oldX) || (mouseY != oldY)) {
			// create the illusion of smooth drawing
			edit.drawLine(view.getColor(),oldX,oldY,mouseX,mouseY);
		} else {
			edit.drawPoint(view.getColor(),mouseX,mouseY);
		}
		
		// don't add duplicate mouse events
		if (!edit.equals(model.getLastEdit())) {
			model.edit(edit);
			model.setLastEdit(edit);
			
			// if we're working on a compound edit, add the edit now
			if (isChanging()) {
				this.compoundEdit.addEdit(edit);
			}
		}
		
		// reset the old mouse cursor to complete the illusion
		this.oldX = mouseX;
		this.oldY = mouseY;
	}
	
	private void lineTool() {}
	private void rectangleTool() {}
	private void fillRectangleTool() {}
	private void ovalTool() {}
	private void fillOvalTool() {}
	private void textTool() {}
	private void fillTool() {}
	private void marqueeTool() {}
	
	private void useTool() {
		int tool = view.getTool();
		
		switch (tool) {
			case SpriteEdit.POINT_TOOL: pointTool(); break;
			case SpriteEdit.LINE_TOOL: lineTool(); break;
			case SpriteEdit.RECTANGLE_TOOL: rectangleTool(); break;
			case SpriteEdit.FILLED_RECTANGLE_TOOL: fillRectangleTool(); break;
			case SpriteEdit.OVAL_TOOL: ovalTool(); break;
			case SpriteEdit.FILLED_OVAL_TOOL: fillOvalTool(); break;
			case SpriteEdit.TEXT_TOOL: textTool(); break;
			case SpriteEdit.FILL_TOOL: fillTool(); break;
			case SpriteEdit.MARQUEE_TOOL: marqueeTool(); break;
		}
	}
	
	/* public methods */
	
	public static boolean isButtonDown(int button, MouseEvent e) {
		int mask = BUTTON_MASK[button];
		
		return ((mask & e.getModifiers()) == mask);
	}
	
	/* event listeners */
	
	// mouse listener interface
	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	
	public void mousePressed(MouseEvent e) {
		int zoom = getZoom();
		
		this.mouseX = (e.getX() / zoom);
		this.mouseY = (e.getY() / zoom);
		
		if (isButtonDown(LEFT_BUTTON,e)) {
			// set the drag spot for mouse event gaps
			this.oldX = mouseX;
			this.oldY = mouseY;
			
			beginChange();
			this.useTool();
		}
	}
	
	public void mouseReleased(MouseEvent e) {
		if (isChanging()) {
			endChange();
		}
	}
	
	// mouse motion listener interface
	public void mouseMoved(MouseEvent e) {}
	
	public void mouseDragged(MouseEvent e) {
		int zoom = getZoom();
		
		this.mouseX = (e.getX() / zoom);
		this.mouseY = (e.getY() / zoom);
		
		if (isButtonDown(LEFT_BUTTON,e)) {
			this.useTool();
		}
	}
}
