/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.model;

import java.awt.Color;

import java.awt.image.BufferedImage;

import java.util.Observable;

import jspritemaker.datastructures.PixelGrid;
import jspritemaker.datastructures.StackList;

import jspritemaker.toolkit.SpriteToolkit;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class Sprite extends Observable implements SpriteToolkit {
	private PixelGrid grid;
	private StackList undoStack, redoStack;
	private String lightPlaneName, darkPlaneName, maskName;
	private int calc;
	
	private SpriteEdit lastEdit;
	
	// class constants
	public final static int CALC_TI89 = 0;
	public final static int CALC_TI92 = 1;
	
	/* constructors */
	
	public Sprite(int width, int height, int calc, String light, String dark, String mask) {
		// initialize the instance variables
		this.grid = new PixelGrid(width,height);
		
		// set calc type
		this.calc = calc;
		
		// create the undo/redo stacks
		this.undoStack = new StackList();
		this.redoStack = new StackList();
		
		this.lastEdit = null;
		
		// set the default variable names
		this.lightPlaneName = light;
		this.darkPlaneName = dark;
		this.maskName = mask;
	}
	
	/* private methods */
	
	private void rebuild() {
		SpriteEdit e;
		int size = this.undoStack.size(), loop;
		
		// reset the pixmap
		this.grid.clearImage();
		
		for (loop = 0; loop < size; loop++) {
			e = (SpriteEdit)this.undoStack.get(loop);
			e.draw(this);
		}
	}
	
	/* public methods */
	
	// overridden from Observable class
	public void notifyObservers() {
		// notify the observers
		this.setChanged();
		super.notifyObservers();
		this.clearChanged();
	}
	
	public BufferedImage getPixmap() {
		return this.grid.getPixmap();
	}
	
	public int getWidth() {
		return this.grid.getWidth();
	}
	
	public int getHeight() {
		return this.grid.getHeight();
	}
	
	public int getCalc() {
		return this.calc;
	}
	
	public SpriteEdit getLastEdit() {
		return this.lastEdit;
	}
	
	public void setLastEdit(SpriteEdit edit) {
		this.lastEdit = edit;
		
		this.notifyObservers();
	}
	
	public String getLightPlaneName() {
		return this.lightPlaneName;
	}
	
	public void setLightPlaneName(String name) {
		this.lightPlaneName = name;
	}
	
	public String getDarkPlaneName() {
		return this.darkPlaneName;
	}
	
	public void setDarkPlaneName(String name) {
		this.darkPlaneName = name;
	}
	
	public String getMaskName() {
		return this.maskName;
	}
	
	public void setMaskName(String name) {
		this.maskName = name;
	}
	
	public void resize(int width, int height) {
		grid.resize(width,height);
		this.lastEdit = null;
	}
	
	public void setProperties(int width, int height, int calc, String light, String dark, String mask) {
		if ((width != getWidth()) || (height != getHeight())) {
			resize(width,height);
		}
		
		this.calc = calc;
		this.lightPlaneName = light;
		this.darkPlaneName = dark;
		this.maskName = mask;
	}
	
	public void edit(SpriteEdit e) {
		e.draw(this);
	}
	
	public void addToUndoStack(SpriteEdit edit) {
		if (!this.redoStack.isEmpty()) {
			this.redoStack.empty();
		}
		
		this.undoStack.push(edit);
	}
	
	public void undo() {
		SpriteEdit e;
		
		if (!this.undoStack.isEmpty()) {
			// remove the edit from the undoStack
			e = (SpriteEdit)this.undoStack.pop();
			
			if (e.canUndo()) {
				e.undo();
				this.redoStack.push(e);
				
				// now rebuild the model using the undo stack
				this.rebuild();
			}
			
			this.notifyObservers();
		}
	}
	
	public void redo() {
		SpriteEdit e;
		
		if (!this.redoStack.isEmpty()) {
			// remove the edit from the redoStack
			e = (SpriteEdit)this.redoStack.pop();
			
			if (e.canRedo()) {
				// are we getting here?
				e.redo();
				this.undoStack.push(e);
				
				// now update the model
				e.draw(this);
			}
			
			this.notifyObservers();
		}
	}
	
	/* interfaces */
	
	// sprite toolkit interface
	public void drawPoint(Color c, int x, int y) {
		this.grid.drawPoint(c,x,y);
	}
	
	public void drawLine(Color c, int x1, int y1, int x2, int y2) {
		this.grid.drawLine(c,x1,y1,x2,y2);
	}
	
	public void drawRectangle(Color c, int x, int y, int width, int height) {
		this.grid.drawRectangle(c,x,y,width,height);
	}
	
	public void fillRectangle(Color c, int x, int y, int width, int height) {
		this.grid.fillRectangle(c,x,y,width,height);
	}
	
	public void drawOval(Color c, int x, int y, int width, int height) {
		this.grid.drawOval(c,x,y,width,height);
	}
	
	public void fillOval(Color c, int x, int y, int width, int height) {
		this.grid.fillOval(c,x,y,width,height);
	}
	
	public void drawText(Color c, String text, int x, int y) {
		this.grid.drawText(c,text,x,y);
	}
	
	public void fill(Color c, int x, int y) {
		this.grid.fill(c,x,y);
	}
	
	public void selectRectangle(int x, int y, int width, int height) {
		// TODO: create selection method
	}
}
