/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.model;

import java.awt.Color;

import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoableEdit;

import jspritemaker.toolkit.SpriteToolkit;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class SpriteEdit implements SpriteToolkit, UndoableEdit {
	private Color c;
	private String text;
	private int type, x, y, x1, y1, x2, y2, width, height;
	
	// undo variables
	protected boolean alive;
	protected boolean done;
	
	// class constants
	public final static int POINT_TOOL            = 1;
	public final static int LINE_TOOL             = 2;
	public final static int RECTANGLE_TOOL        = 3;
	public final static int FILLED_RECTANGLE_TOOL = 4;
	public final static int OVAL_TOOL             = 5;
	public final static int FILLED_OVAL_TOOL      = 6;
	public final static int TEXT_TOOL             = 7;
	public final static int FILL_TOOL             = 8;
	public final static int MARQUEE_TOOL          = 9;
	
	/* constructors */
	
	public SpriteEdit() {
		// set all type tool variables to blank
		this.type = 0;
		this.c = null;
		this.text = null;
		this.x = -1;
		this.y = -1;
		this.x2 = -1;
		this.y2 = -1;
		this.width = -1;
		this.height = -1;
		
		// initialize undo variables
		this.alive = true;
		this.done = false;
	}
	
	/* public methods */
	
	public int getType() {
		return this.type;
	}
	
	public Color getColor() {
		return this.c;
	}
	
	public String getText() {
		return this.text;
	}
	
	public int getX() {
		return this.x;
	}
	
	public int getY() {
		return this.y;
	}
	
	public int getX2() {
		return this.x2;
	}
	
	public int getY2() {
		return this.y2;
	}
	
	public int getWidth() {
		return this.width;
	}
	
	public int getHeight() {
		return this.height;
	}
	
	// overridden from Object class
	public boolean equals(Object obj) {
		SpriteEdit edit;
		boolean equal = false, colorTest = false, textTest = false;
		Color c;
		String text;
		
		if (obj instanceof SpriteEdit) {
			edit = (SpriteEdit)obj;
			
			if ((edit.getType() == type) && (edit.getX() == x) && (edit.getY() == y) &&
			    (edit.getX2() == x2) && (edit.getY2() == y2) &&
			    (edit.getWidth() == width) && (edit.getHeight() == height)) {
				if ((this.c == null) && (edit.getColor() == null)) {
					colorTest = true;
				} else if ((this.c != null) && (edit.getColor() != null)) {
					if (edit.getColor().equals(this.c)) {
						colorTest = true;
					}
				}
				
				if ((this.text == null) && (edit.getText() == null)) {
					textTest = true;
				} else if ((this.text != null) && (edit.getText() != null)) {
					if (edit.getText().equals(this.text)) {
						textTest = true;
					}
				}
				
				equal = (colorTest && textTest);
			}
			
		}
		
		return equal;
	}
	
	public void draw(SpriteToolkit tk) {		
		switch (this.type) {
			case POINT_TOOL: tk.drawPoint(c,x,y); break;
			case LINE_TOOL: tk.drawLine(c,x,y,x2,y2); break;
			case RECTANGLE_TOOL: tk.drawRectangle(c,x,y,width,height); break;
			case FILLED_RECTANGLE_TOOL: tk.fillRectangle(c,x,y,width,height); break;
			case OVAL_TOOL: tk.drawOval(c,x,y,width,height); break;
			case FILLED_OVAL_TOOL: tk.fillOval(c,x,y,width,height); break;
			case TEXT_TOOL: tk.drawText(c,text,x,y); break;
			case FILL_TOOL: tk.fill(c,x,y); break;
			case MARQUEE_TOOL: tk.selectRectangle(x,y,width,height); break;
			
			// last case statement MUST be ABOVE this line
			
			// this case should never be reached!
			default: throw new UnsupportedOperationException("Invalid Tool Type " + type);
		}
	}
	
	/* interfaces */
	
	// sprite toolkit interface
	public void drawPoint(Color c, int x, int y) {
		this.c = c;
		this.x = x;
		this.y = y;
		this.type = POINT_TOOL;
		
		// change the model
		this.redo();
	}
	
	public void drawLine(Color c, int x1, int y1, int x2, int y2) {
		this.c = c;
		this.x = x1;
		this.y = y1;
		this.x2 = x2;
		this.y2 = y2;
		this.type = LINE_TOOL;
		
		// change the model
		this.redo();
	}
	
	public void drawRectangle(Color c, int x, int y, int width, int height) {
		this.c = c;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.type = RECTANGLE_TOOL;
		
		// change the model
		this.redo();
	}
	
	public void fillRectangle(Color c, int x, int y, int width, int height) {
		this.c = c;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.type = FILLED_RECTANGLE_TOOL;
		
		// change the model
		this.redo();
	}
	
	public void drawOval(Color c, int x, int y, int width, int height) {
		this.c = c;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.type = OVAL_TOOL;
		
		// change the model
		this.redo();
	}
	
	public void fillOval(Color c, int x, int y, int width, int height) {
		this.c = c;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.type = FILLED_OVAL_TOOL;
		
		// change the model
		this.redo();
	}
	
	public void drawText(Color c, String text, int x, int y) {
		this.c = c;
		this.text = text;
		this.x = x;
		this.y = y;
		this.type = TEXT_TOOL;
		
		// change the model
		this.redo();
	}
	
	public void fill(Color c, int x, int y) {
		this.c = c;
		this.x = x;
		this.y = y;
		this.type = FILL_TOOL;
		
		// change the model
		this.redo();
	}
	
	public void selectRectangle(int x, int y, int width, int height) {
		this.c = c;
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.type = MARQUEE_TOOL;
		
		// change the model
		this.redo();
	}
	
	// undoable edit interface
	public boolean isSignificant() {
		return false;
	}
	
	public boolean addEdit(UndoableEdit e) {
		return false;
	}
	
	public boolean replaceEdit(UndoableEdit e) {
		return false;
	}
	
	public void die() {
		this.alive = false;
	}
	
	public boolean canRedo() {
		return (alive && !done);
	}
	
	public boolean canUndo() {
		return (alive && done);
	}
	
	public void redo() throws CannotRedoException {
		if (!canRedo()) {
			throw new CannotRedoException();
		}
		
		this.done = true;
	}
	
	public void undo() throws CannotUndoException {
		if (!canUndo()) {
			throw new CannotUndoException();
		}
		
		this.done = false;
	}
	
	// these methods should not be used
	public String getPresentationName() {
		throw new UnsupportedOperationException();
	}
	
	public String getRedoPresentationName() {
		throw new UnsupportedOperationException();
	}
	
	public String getUndoPresentationName() {
		throw new UnsupportedOperationException();
	}
}
