/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 */
package jspritemaker.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import java.util.Observer;
import java.util.Observable;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import javax.swing.event.InternalFrameListener;
import javax.swing.event.InternalFrameEvent;

import jspritemaker.controller.CodePaneInput;
import jspritemaker.controller.ColorChooserInput;
import jspritemaker.controller.SpritePanelInput;

import jspritemaker.model.Sprite;
import jspritemaker.model.SpriteEdit;

import jspritemaker.toolkit.SpriteToolkit;

/**
 * JSpriteMaker 68k
 * Version 1.0
 * Copyright (C) 2002 John David Ratliff
 */
public class SpriteView extends JInternalFrame implements Observer, InternalFrameListener, SpriteToolkit {
	private SpriteManager owner;
	private SpritePanel panel;
	private ZoomControl zoomSlider;
	private ColorChooser color;
	private CodeView code;
	private Sprite sprite;
	private int zoom;
	
	// class constants
	public final static int OFFSET = 15;
	
	public final static int MIN_ZOOM = 1;
	public final static int MAX_ZOOM = 50;
	
	/* constructors */
	
	protected SpriteView(SpriteManager owner, Sprite sprite, int count) {
		super("New Sprite " + count,true,true,true,true);
		
		this.owner = owner;
		this.sprite = sprite;
		
		// initialize the class variables
		this.zoom = 2;
		
		// observe the model for changes
		sprite.addObserver(this);
		
		// initialize the frame
		this.initFrame(count);
	}
	
	/* private methods */
	
	private void initFrame(int count) {
		int cascade = (OFFSET * (count - 1));
		
		// add event listeners
		this.addInternalFrameListener(this);
		
		// setup the layout
		this.getContentPane().add(buildContentPane());

		// setup frame properties
		this.setSize(getPreferredSize());
		this.setLocation(cascade,cascade);
		
		// finally, make the frame visible
		this.setVisible(true);
	}
	
	private JScrollPane buildContentPane() {
		JScrollPane pane = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel panel = new JPanel(layout);
		
		// create the sprite panel
		this.panel = new SpritePanel(this,this.sprite);
		
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 1;
		gbc.gridheight = 2;
		gbc.weightx = 1.0;
		gbc.weighty = 0.0;
		gbc.insets = new Insets(10,10,10,10);
		gbc.anchor = GridBagConstraints.NORTHWEST;
		layout.setConstraints(this.panel,gbc);
		panel.add(this.panel);
		
		// create the code view
		this.code = new CodeView(sprite);
		
		gbc.gridx = 1;
		layout.setConstraints(this.code,gbc);
		panel.add(code);
		
		// create the slider control
		this.zoomSlider = new ZoomControl(this);
		
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.weighty = 1.0;
		gbc.gridheight = 1;
		layout.setConstraints(this.zoomSlider,gbc);
		panel.add(zoomSlider);
		
		// create the color chooser
		this.color = new ColorChooser(this);
		
		gbc.gridx = 1;
		layout.setConstraints(this.color,gbc);
		panel.add(color);
		
		// create the scroll pane
		pane.setViewportView(panel);
		
		return pane;
	}
	
	// Java 2 1.3 JSlider Bug fix
	// code based on Brian Clark's code on Java Developer Connection for
	// JSlider/JInternalFrame bug -- bug fixed in JRE 1.4
	// http://developer.java.sun.com/developer/bugParade/bugs/4220108.html
	private void sliderFixHack() {
		Dimension d = this.zoomSlider.getSize();
		
		d.height++;
		zoomSlider.setSize(d);
		zoomSlider.repaint();
		
		d.height--;
		zoomSlider.setSize(d);
		zoomSlider.repaint();
	}
	
	/* protected methods */
	
	protected Sprite getSprite() {
		return this.sprite;
	}
	
	protected void setSprite(Sprite sprite) {
		this.sprite = sprite;
	}
	
	protected void setPanelInput(SpritePanelInput mouse) {
		this.panel.addMouseListener(mouse);
		this.panel.addMouseMotionListener(mouse);
	}
	
	protected void setColorInput(ColorChooserInput mouse) {
		this.color.addMouseListener(mouse);
		this.color.addMouseMotionListener(mouse);
	}
	
	protected void setCodeInput(CodePaneInput input) {
		this.code.setCodeInput(input);
	}
	
	protected void changeZoom(int zoom) {
		if (zoom < MIN_ZOOM) {
			zoom = MIN_ZOOM;
		} else if (zoom > MAX_ZOOM) {
			zoom = MAX_ZOOM;
		}
		
		this.zoom = zoom;
		this.panel.redraw();
		this.validate();
	}
	
	/* public methods */
	
	public Dimension getPreferredSize() {
		return new Dimension(640,480);
	}
	
	public int getZoom() {
		return this.zoom;
	}
	
	public void setZoom(int zoom) {
		this.zoomSlider.setValue(zoom);
	}
	
	public Color getColor() {
		return this.color.getColor();
	}
	
	public void setColor(int color) {
		this.color.setColor(color);
	}
	
	public void copy() {
		this.code.copy();
	}
	
	public void selectAll() {
		this.code.selectAll();
	}
	
	public void resize() {
		this.panel.redraw();
		this.validate();
	}
	
	public void disappearingEdit(SpriteEdit edit) {
		edit.draw(this);
	}
	
	/* interfaces */
	
	// observer interface
	public void update(Observable o, Object arg) {
		this.panel.update();
		this.code.update();
	}
	
	// sprite toolkit interface
	public void drawPoint(Color c, int x, int y) {}	
	public void drawLine(Color c, int x1, int y1, int x2, int y2) {}	
	public void drawRectangle(Color c, int x, int y, int width, int height) {}
	public void fillRectangle(Color c, int x, int y, int width, int height) {}
	public void drawOval(Color c, int x, int y, int width, int height) {}
	public void fillOval(Color c, int x, int y, int width, int height) {}
	public void drawText(Color c, String text, int x, int y) {}
	public void fill(Color c, int x, int y) {}
	public void selectRectangle(int x, int y, int width, int height) {}
	
	/* event listeners*/
	
	// internal frame listener interface
	public void internalFrameClosed(InternalFrameEvent e) {}
	public void internalFrameDeactivated(InternalFrameEvent e) {}
	public void internalFrameDeiconified(InternalFrameEvent e) {}
	public void internalFrameIconified(InternalFrameEvent e) {}
	public void internalFrameOpened(InternalFrameEvent e) {}
	
	public void internalFrameActivated(InternalFrameEvent e) {
		owner.setCurrentSprite(this);
		sliderFixHack();
	}
	
	public void internalFrameClosing(InternalFrameEvent e) {
		if (this.isSelected()) {
			owner.setCurrentSprite(null);
		}
	}
}
